-- AsyncTradesTest.lua
running = true                	-- Script operation flag
TRADE_ACC="4100R24"       	    -- A trading account in the QUIK trade server
CLASS_CODE_FUT="SPBFUT"       	-- FUTURES class
SEC_CODE_FUT_FOR_OPEN="SiU6"  	-- Security (futures) code
TRANS_ID=0                    	-- Transaction ID set by a user when sending a transaction from QLUA
-- 
local logfile = 0 				-- A file for logging
local start_test,finish_test;	-- Text beginning and end
local transIDstart				-- The starting transaction ID is set based on the minute and second
local tradecounter              -- Counter of performed trades

resultinfo=""
running=true
finished=false
tickcounter=0
-- This is the main function if the script, runs automatically; the script runs until the function completes 
function main()
	-- Tries to open a file in the "read/write" mode
	logfile = io.open(getScriptPath().."\\AsyncTradesTest_log.txt","a+");
	-- If the file does not exist
	if logfile == nil then 
		-- Creates a file in the "write" mode
		logfile = io.open(getScriptPath().."\\AsyncTradesTest_log.txt","w"); 
		-- Closes the file
		logfile:close();
		-- Opens an existing file and the "read/write" mode
		logfile = io.open(getScriptPath().."\\AsyncTradesTest_log.txt","r+");
	end;
	-- Setting the number of the first transaction
	minutes=os.date("%M")
	seconds=os.date("%S")
	transIDstart=minutes*10000+seconds*100	
	Print("AsyncTradesTest.lua starting...");	
	tradecounter=0
   -- Fills a structure to send a transaction to buy 1 lot 
    local Transaction={
      ["TRANS_ID"]   = tostring(TRANS_ID),
      ["ACTION"]     = "NEW_ORDER",
      ["CLASSCODE"]  = CLASS_CODE_FUT,
      ["SECCODE"]    = SEC_CODE_FUT_FOR_OPEN,
      ["OPERATION"]  = "B", -- BUY
      ["TYPE"]       = "M", -- MARKET
      ["QUANTITY"]   = "1", -- amount
      ["ACCOUNT"]    = TRADE_ACC,
      ["PRICE"]      = tostring(getParamEx(CLASS_CODE_FUT, SEC_CODE_FUT_FOR_OPEN, "offer").param_value + 30*getParamEx(CLASS_CODE_FUT, SEC_CODE_FUT_FOR_OPEN, "SEC_PRICE_STEP").param_value), -- at a price 10 minimal steps higher
      ["COMMENT"]    = "Test"
    }	
	tradecounter=0;
	-- Remember the start time
	start_test=os.clock()
	-- Running a series of 10 Buy transactions
	for trade = 1, 10, 1 do
        -- Replacing Buy() with a direct call 
		tradecounter=tradecounter+1;
		transIDstart=transIDstart+1
		TRANS_ID = transIDstart		
		Transaction["TRANS_ID"]=tostring(TRANS_ID);
		Transaction["COMMENT"]=tostring(tradecounter);
		local Result = sendTransaction(Transaction);
		-- Checking the result of sending
		if Result ~= "" then
			message("Buying failed!\nERROR: "..tostring(Result));	
		end; 
	    Print((tradecounter)..". Buy 1 lot "..SEC_CODE_FUT_FOR_OPEN.." TRANS_ID="..TRANS_ID)   		
    end
	-- Test completed
	finish_test=os.clock()
	fulltime=(finish_test-start_test)*1000.
	finished=true
	info=string.format("Total time=%.1f ms",fulltime)
	Print(info)
	info=string.format("Average time=%.2f ms (%d series)",fulltime/tradecounter,tradecounter)
	Print(info)
	resultinfo=string.format("Average time=%.2f ms (%d series)",fulltime/tradecounter, tradecounter)	
	Print("AsyncTradesTest.lua finishing...");	
	-- Script stop flag 
    while running do
        sleep(1000)
    end
    -- Resetting the file
	logfile:flush();
	logfile:close();	
end

-- Called with each update of the order book
function OnQuote(class, sec ) 
	-- Creating a timer of the number of received ticks in order to stop the script after completing trading operations
	if (finished) then             -- If the series of trading operations has completed
		tickcounter=tickcounter+1; -- start to count order book updates
		if(tickcounter>5) then     -- exit after the fifth tick in the market depth
			message(resultinfo);   -- But first show test results
			running=false          -- Stop
		end	
	end
end

-- The event of script stop by user
function OnStop(s)
	-- Setting the flag to 'Off'
	running = false
	Print("finishing...");
	message(resultinfo)
	return 3000 -- Setting timeout of 3 seconds
end

function Print(line)
	if (logfile~=nil) then
	    timestamp=os.date("%Y.%m.%d %H:%M:%S ");
		logfile:write(timestamp.."\t"..line.."\n");
	end
end
-- Buying the instrument
function Buy()
	transIDstart=transIDstart+1
    TRANS_ID = transIDstart
	tradecounter=tradecounter+1
	Print((tradecounter)..". Buy 1 lot "..SEC_CODE_FUT_FOR_OPEN.." TRANS_ID="..TRANS_ID)
   -- Fills a structure to send a transaction to buy 1 lot 
    local Transaction={
      ["TRANS_ID"]   = tostring(TRANS_ID),
      ["ACTION"]     = "NEW_ORDER",
      ["CLASSCODE"]  = CLASS_CODE_FUT,
      ["SECCODE"]    = SEC_CODE_FUT_FOR_OPEN,
      ["OPERATION"]  = "B", -- BUY
      ["TYPE"]       = "M", -- MARKET
      ["QUANTITY"]   = "1", -- amount
      ["ACCOUNT"]    = TRADE_ACC,
      ["PRICE"]      = tostring(getParamEx(CLASS_CODE_FUT, SEC_CODE_FUT_FOR_OPEN, "offer").param_value + 30*getParamEx(CLASS_CODE_FUT, SEC_CODE_FUT_FOR_OPEN, "SEC_PRICE_STEP").param_value), -- at a price 10 minimal steps higher
      ["COMMENT"]    = string.format("Trade #d Buy %s at %s",(tradecounter),SEC_CODE_FUT_FOR_OPEN,os.date("%Y.%m.%d %H:%M:%S"))
    }
    -- At what price to buy
        local price=getParamEx(CLASS_CODE_FUT, SEC_CODE_FUT_FOR_OPEN, "offer").param_value
    --message("Offer Price="..price)
    -- Time of sending a trade transaction to a server
	topen_start=os.clock();  -- The time of buying
	last_operation=BUY
    local Result = sendTransaction(Transaction);
    -- Checking correctness
	if Result ~= "" then
		message("Buying failed!\nERROR: "..tostring(Result));
    end;    
end
